@@ -6,6 +6,7 @@
 
 use std::error::Error;
 
+use async_trait::async_trait;
 use bytes::Bytes;
 use http::request::Builder as RequestBuilder;
 use http::Response;
@@ -30,3 +31,22 @@ pub trait Client {
         body: Vec<u8>,
     ) -> Result<Response<Bytes>, ApiError<Self::Error>>;
 }
+
+/// A trait representing an asynchronous client which can communicate with a GitLab instance.
+#[async_trait]
+pub trait AsyncClient {
+    /// The errors which may occur for this client.
+    type Error: Error + Send + Sync + 'static;
+
+    /// Get the URL for the endpoint for the client.
+    ///
+    /// This method adds the hostname for the client's target instance.
+    fn rest_endpoint(&self, endpoint: &str) -> Result<Url, ApiError<Self::Error>>;
+
+    /// Send a REST query asynchronously.
+    async fn rest_async(
+        &self,
+        request: RequestBuilder,
+        body: Vec<u8>,
+    ) -> Result<Response<Bytes>, ApiError<Self::Error>>;
+}
