@@ -12,6 +12,9 @@ mod create;
 mod edit;
 mod issue;
 mod issues;
+mod merge_requests_closing;
+pub mod notes;
+mod resource_label_events;
 
 pub use self::create::CreateIssue;
 pub use self::create::CreateIssueBuilder;
@@ -23,9 +26,18 @@ pub use self::edit::IssueStateEvent;
 pub use self::issue::Issue;
 pub use self::issue::IssueBuilder;
 
+pub use self::issues::IssueDueDateFilter;
+pub use self::issues::IssueIteration;
 pub use self::issues::IssueOrderBy;
 pub use self::issues::IssueScope;
+pub use self::issues::IssueSearchScope;
 pub use self::issues::IssueState;
 pub use self::issues::IssueWeight;
 pub use self::issues::Issues;
 pub use self::issues::IssuesBuilder;
+
+pub use self::merge_requests_closing::MergeRequestsClosing;
+pub use self::merge_requests_closing::MergeRequestsClosingBuilder;
+
+pub use self::resource_label_events::IssueResourceLabelEvents;
+pub use self::resource_label_events::IssueResourceLabelEventsBuilder;
