@@ -4,28 +4,26 @@
 // option. This file may not be copied, modified, or distributed
 // except according to those terms.
 
-use std::collections::BTreeSet;
 use std::iter;
 
 use derive_builder::Builder;
-use itertools::Itertools;
 
-use crate::api::common::NameOrId;
+use crate::api::common::{CommaSeparatedList, NameOrId};
 use crate::api::endpoint_prelude::*;
-use crate::api::projects::merge_requests::create::Assignee;
+use crate::api::projects::merge_requests::create::{Assignee, Reviewer};
 use crate::api::ParamValue;
 
 #[derive(Debug, Clone)]
 enum MergeRequestLabels<'a> {
     Unlabeled,
-    Labeled(BTreeSet<Cow<'a, str>>),
+    Labeled(CommaSeparatedList<Cow<'a, str>>),
 }
 
 impl<'a, 'b: 'a> ParamValue<'a> for &'b MergeRequestLabels<'a> {
-    fn as_value(self) -> Cow<'a, str> {
+    fn as_value(&self) -> Cow<'a, str> {
         match self {
             MergeRequestLabels::Unlabeled => "".into(),
-            MergeRequestLabels::Labeled(labels) => format!("{}", labels.iter().format(",")).into(),
+            MergeRequestLabels::Labeled(labels) => format!("{}", labels).into(),
         }
     }
 }
@@ -49,7 +47,7 @@ impl MergeRequestStateEvent {
 }
 
 impl ParamValue<'static> for MergeRequestStateEvent {
-    fn as_value(self) -> Cow<'static, str> {
+    fn as_value(&self) -> Cow<'static, str> {
         self.as_str().into()
     }
 }
@@ -73,12 +71,18 @@ pub struct EditMergeRequest<'a> {
     /// The assignee of the merge request.
     #[builder(setter(name = "_assignee"), default, private)]
     assignee: Option<Assignee>,
+    #[builder(setter(name = "_reviewer"), default, private)]
+    reviewer: Option<Reviewer>,
     /// The ID of the milestone to add the merge request to.
     #[builder(default)]
     milestone_id: Option<u64>,
     /// Labels to add to the merge request.
     #[builder(setter(name = "_labels"), default, private)]
     labels: Option<MergeRequestLabels<'a>>,
+    #[builder(setter(name = "_add_labels"), default, private)]
+    add_labels: Option<CommaSeparatedList<Cow<'a, str>>>,
+    #[builder(setter(name = "_remove_labels"), default, private)]
+    remove_labels: Option<CommaSeparatedList<Cow<'a, str>>>,
     /// The description of the merge request.
     #[builder(setter(into), default)]
     description: Option<Cow<'a, str>>,
@@ -112,13 +116,13 @@ impl<'a> EditMergeRequest<'a> {
 }
 
 impl<'a> EditMergeRequestBuilder<'a> {
-    /// Filter unassigned merge requests.
+    /// Unset assignee(s) of merge request
     pub fn unassigned(&mut self) -> &mut Self {
         self.assignee = Some(Some(Assignee::Unassigned));
         self
     }
 
-    /// Filter merge requests assigned to a user (by ID).
+    /// Assign merge request to a user (by ID).
     pub fn assignee(&mut self, assignee: u64) -> &mut Self {
         let assignee = match self.assignee.take() {
             Some(Some(Assignee::Ids(mut set))) => {
@@ -135,7 +139,7 @@ impl<'a> EditMergeRequestBuilder<'a> {
         self
     }
 
-    /// Filter merge requests assigned to a user (by ID).
+    /// Assign merge request to users (by ID).
     pub fn assignees<I>(&mut self, iter: I) -> &mut Self
     where
         I: Iterator<Item = u64>,
@@ -155,31 +159,70 @@ impl<'a> EditMergeRequestBuilder<'a> {
         self
     }
 
-    /// Clear all labels
+    /// Filter merge requests without a reviewer.
+    pub fn without_reviewer(&mut self) -> &mut Self {
+        self.reviewer = Some(Some(Reviewer::Unassigned));
+        self
+    }
+
+    /// Filter merge requests reviewed by a user (by ID).
+    pub fn reviewer(&mut self, reviewer: u64) -> &mut Self {
+        let reviewer = match self.reviewer.take() {
+            Some(Some(Reviewer::Ids(mut set))) => {
+                set.insert(reviewer);
+                Reviewer::Ids(set)
+            },
+            _ => Reviewer::Ids(iter::once(reviewer).collect()),
+        };
+        self.reviewer = Some(Some(reviewer));
+        self
+    }
+
+    /// Filter merge requests reviewed by users (by ID).
+    pub fn reviewers<I>(&mut self, iter: I) -> &mut Self
+    where
+        I: Iterator<Item = u64>,
+    {
+        let reviewer = match self.reviewer.take() {
+            Some(Some(Reviewer::Ids(mut set))) => {
+                set.extend(iter);
+                Reviewer::Ids(set)
+            },
+            _ => Reviewer::Ids(iter.collect()),
+        };
+        self.reviewer = Some(Some(reviewer));
+        self
+    }
+
+    /// Remove all labels from the issue.
+    #[deprecated(note = "use `clear_labels` instead")]
     pub fn remove_labels(&mut self) -> &mut Self {
+        self.clear_labels()
+    }
+
+    /// Remove all labels from the issue.
+    pub fn clear_labels(&mut self) -> &mut Self {
         self.labels = Some(Some(MergeRequestLabels::Unlabeled));
         self
     }
 
-    /// Add a label.
+    /// Add a label
     pub fn label<L>(&mut self, label: L) -> &mut Self
     where
         L: Into<Cow<'a, str>>,
     {
         let label = label.into();
         let labels = if let Some(Some(MergeRequestLabels::Labeled(mut set))) = self.labels.take() {
-            set.insert(label);
+            set.push(label);
             set
         } else {
-            let mut set = BTreeSet::new();
-            set.insert(label);
-            set
+            iter::once(label).collect()
         };
         self.labels = Some(Some(MergeRequestLabels::Labeled(labels)));
         self
     }
 
-    /// Add multiple labels.
+    /// Add multiple labels
     pub fn labels<I, L>(&mut self, iter: I) -> &mut Self
     where
         I: Iterator<Item = L>,
@@ -195,6 +238,34 @@ impl<'a> EditMergeRequestBuilder<'a> {
         self.labels = Some(Some(MergeRequestLabels::Labeled(labels)));
         self
     }
+
+    /// Add a label to the merge request.
+    ///
+    /// This is an incremental addition to the existing set of labels on the merge request.
+    pub fn add_label<L>(&mut self, label: L) -> &mut Self
+    where
+        L: Into<Cow<'a, str>>,
+    {
+        self.add_labels
+            .get_or_insert(None)
+            .get_or_insert_with(CommaSeparatedList::new)
+            .push(label.into());
+        self
+    }
+
+    /// Remove a label from the merge request.
+    ///
+    /// This is an incremental remove form the existing set of labels on the merge request.
+    pub fn remove_label<L>(&mut self, label: L) -> &mut Self
+    where
+        L: Into<Cow<'a, str>>,
+    {
+        self.remove_labels
+            .get_or_insert(None)
+            .get_or_insert_with(CommaSeparatedList::new)
+            .push(label.into());
+        self
+    }
 }
 
 impl<'a> Endpoint for EditMergeRequest<'a> {
@@ -218,6 +289,8 @@ impl<'a> Endpoint for EditMergeRequest<'a> {
             .push_opt("title", self.title.as_ref())
             .push_opt("milestone_id", self.milestone_id)
             .push_opt("labels", self.labels.as_ref())
+            .push_opt("add_labels", self.add_labels.as_ref())
+            .push_opt("remove_labels", self.remove_labels.as_ref())
             .push_opt("description", self.description.as_ref())
             .push_opt("state_event", self.state_event)
             .push_opt("remove_source_branch", self.remove_source_branch)
@@ -228,6 +301,9 @@ impl<'a> Endpoint for EditMergeRequest<'a> {
         if let Some(assignee) = self.assignee.as_ref() {
             assignee.add_params(&mut params);
         }
+        if let Some(reviewer) = self.reviewer.as_ref() {
+            reviewer.add_params(&mut params);
+        }
 
         #[allow(deprecated)]
         {
@@ -408,6 +484,67 @@ mod tests {
         api::ignore(endpoint).query(&client).unwrap();
     }
 
+    #[test]
+    fn endpoint_unreviewed() {
+        let endpoint = ExpectedUrl::builder()
+            .method(Method::PUT)
+            .endpoint("projects/simple%2Fproject/merge_requests/1")
+            .content_type("application/x-www-form-urlencoded")
+            .body_str("reviewer_ids=0")
+            .build()
+            .unwrap();
+        let client = SingleTestClient::new_raw(endpoint, "");
+
+        let endpoint = EditMergeRequest::builder()
+            .project("simple/project")
+            .merge_request(1)
+            .without_reviewer()
+            .build()
+            .unwrap();
+        api::ignore(endpoint).query(&client).unwrap();
+    }
+
+    #[test]
+    fn endpoint_reviewer() {
+        let endpoint = ExpectedUrl::builder()
+            .method(Method::PUT)
+            .endpoint("projects/simple%2Fproject/merge_requests/1")
+            .content_type("application/x-www-form-urlencoded")
+            .body_str("reviewer_ids%5B%5D=1")
+            .build()
+            .unwrap();
+        let client = SingleTestClient::new_raw(endpoint, "");
+
+        let endpoint = EditMergeRequest::builder()
+            .project("simple/project")
+            .merge_request(1)
+            .reviewer(1)
+            .build()
+            .unwrap();
+        api::ignore(endpoint).query(&client).unwrap();
+    }
+
+    #[test]
+    fn endpoint_reviewers() {
+        let endpoint = ExpectedUrl::builder()
+            .method(Method::PUT)
+            .endpoint("projects/simple%2Fproject/merge_requests/1")
+            .content_type("application/x-www-form-urlencoded")
+            .body_str(concat!("reviewer_ids%5B%5D=1", "&reviewer_ids%5B%5D=2"))
+            .build()
+            .unwrap();
+        let client = SingleTestClient::new_raw(endpoint, "");
+
+        let endpoint = EditMergeRequest::builder()
+            .project("simple/project")
+            .merge_request(1)
+            .reviewer(1)
+            .reviewers([1, 2].iter().copied())
+            .build()
+            .unwrap();
+        api::ignore(endpoint).query(&client).unwrap();
+    }
+
     #[test]
     fn endpoint_milestone_id() {
         let endpoint = ExpectedUrl::builder()
@@ -450,6 +587,7 @@ mod tests {
     }
 
     #[test]
+    #[allow(deprecated)]
     fn endpoint_labels_remove() {
         let endpoint = ExpectedUrl::builder()
             .method(Method::PUT)
@@ -469,6 +607,68 @@ mod tests {
         api::ignore(endpoint).query(&client).unwrap();
     }
 
+    #[test]
+    fn endpoint_labels_clear() {
+        let endpoint = ExpectedUrl::builder()
+            .method(Method::PUT)
+            .endpoint("projects/simple%2Fproject/merge_requests/1")
+            .content_type("application/x-www-form-urlencoded")
+            .body_str("labels=")
+            .build()
+            .unwrap();
+        let client = SingleTestClient::new_raw(endpoint, "");
+
+        let endpoint = EditMergeRequest::builder()
+            .project("simple/project")
+            .merge_request(1)
+            .clear_labels()
+            .build()
+            .unwrap();
+        api::ignore(endpoint).query(&client).unwrap();
+    }
+
+    #[test]
+    fn endpoint_add_labels() {
+        let endpoint = ExpectedUrl::builder()
+            .method(Method::PUT)
+            .endpoint("projects/simple%2Fproject/merge_requests/1")
+            .content_type("application/x-www-form-urlencoded")
+            .body_str("add_labels=one%2Ctwo")
+            .build()
+            .unwrap();
+        let client = SingleTestClient::new_raw(endpoint, "");
+
+        let endpoint = EditMergeRequest::builder()
+            .project("simple/project")
+            .merge_request(1)
+            .add_label("one")
+            .add_label("two")
+            .build()
+            .unwrap();
+        api::ignore(endpoint).query(&client).unwrap();
+    }
+
+    #[test]
+    fn endpoint_remove_labels() {
+        let endpoint = ExpectedUrl::builder()
+            .method(Method::PUT)
+            .endpoint("projects/simple%2Fproject/merge_requests/1")
+            .content_type("application/x-www-form-urlencoded")
+            .body_str("remove_labels=one%2Ctwo")
+            .build()
+            .unwrap();
+        let client = SingleTestClient::new_raw(endpoint, "");
+
+        let endpoint = EditMergeRequest::builder()
+            .project("simple/project")
+            .merge_request(1)
+            .remove_label("one")
+            .remove_label("two")
+            .build()
+            .unwrap();
+        api::ignore(endpoint).query(&client).unwrap();
+    }
+
     #[test]
     fn endpoint_description() {
         let endpoint = ExpectedUrl::builder()
